@@ -19,15 +19,13 @@ use strict;
 use vars qw($VERSION %sigmaLensTypes);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.28';
+$VERSION = '1.34';
 
 # sigma LensType lookup (ref IB)
 %sigmaLensTypes = (
     Notes => q{
         Sigma LensType values are hexadecimal numbers stored as a string (without
-        the leading "0x").  Decimal values have been added to differentiate lenses
-        which would otherwise have the same LensType, and are used by the Composite
-        LensID tag when attempting to identify the specific lens model.
+        the leading "0x").
     },
     # 0x0 => 'Sigma 50mm F2.8 EX Macro', (0x0 used for other lenses too)
     # 0x8 - 18-125mm LENSARANGE@18mm=22-4
@@ -225,6 +223,37 @@ $VERSION = '1.28';
     0x1007 => 'Sigma 30mm F2.8', #PH (DP2 Quattro kit)
     0x1008 => 'Sigma 50mm F2.8 Macro', #NJ (DP3 Quattro kit)
     0x1009 => 'Sigma 14mm F4', #NJ (DP0 Quattro kit)
+    # L-mount lenses?:
+    0x4001 => 'Lumix S 24-105mm F4 Macro OIS (S-R24105)', #IB
+    0x4002 => 'Lumix S 70-200mm F4 OIS (S-R70200)', #IB
+    0x4003 => 'Lumix S 50mm F1.4 (S-X50)', #IB
+    0x4006 => 'Lumix S 24-70mm F2.8 (S-E2470)', #IB
+    0x4007 => 'Lumix S 16-35mm F4 (S-R1635)', #IB
+    0x4008 => 'Lumix S 70-200mm F2.8 OIS (S-E70200)', #IB
+    0x4010 => 'Lumix S 35mm F1.8 (S-S35)', #IB
+    0x4011 => 'LUMIX S 18mm F1.8 (S-S18)', #IB
+    0x400b => 'Lumix S 20-60mm F3.5-5.6 (S-R2060)', #IB
+    0x400c => 'Lumix S 85mm F1.8 (S-S85)', #IB
+    0x400d => 'Lumix S 70-300 F4.5-5.6 Macro OIS (S-R70300)', #IB
+    0x400f => 'Lumix S 24mm F1.8 (S-S24)', #IB
+    0x6001 => 'Sigma 150-600mm F5-6.3 DG OS HSM | S', #PH (NC, fp)
+    0x6003 => 'Sigma 45mm F2.8 DG DN | C', #PH (NC, fp)
+    0x6005 => 'Sigma 14-24mm F2.8 DG DN | A', #IB
+    0x6006 => 'Sigma 50mm F1.4 DG HSM | A', #IB (014)
+    0x6011 => 'Sigma 24-70mm F2.8 DG DN | A', #IB
+    0x6012 => 'Sigma 100-400mm F5-6.3 DG DN OS | C', #IB
+    0x6013 => 'Sigma 100-400mm F5-6.3 DG DN OS | C + TC-1411', #IB
+    0x6015 => 'Sigma 85mm F1.4 DG DN | A', #IB
+    0x6017 => 'Sigma 65mm F2 DG DN | C', #IB
+    0x6018 => 'Sigma 35mm F2 DG DN | C', #IB
+    0x601a => 'Sigma 28-70mm F2.8 DG DN | C', #IB
+    0x601b => 'Sigma 150-600mm F5-6.3 DG DN OS | S', #IB
+    0x6020 => 'Sigma 35mm F1.4 DG DN | A', #IB
+    0x6021 => 'Sigma 90mm F2.8 DG DN | C', #IB
+    0x6023 => 'Sigma 20mm F2 DG DN | C', #IB
+    0x6025 => 'Sigma 20mm F1.4 DG DN | A', #IB
+    0x6026 => 'Sigma 24mm F1.4 DG DN | A', #IB
+    0x602c => "Sigma 50mm F1.4 DG DN | A (2023)", #IB
     0x8005 => 'Sigma 35mm F1.4 DG HSM | A', #PH (012)
     0x8009 => 'Sigma 18-35mm F1.8 DC HSM | A', #PH
     0x8900 => 'Sigma 70-300mm F4-5.6 DG OS', #PH (SD15)
@@ -431,7 +460,7 @@ $VERSION = '1.28';
     0x001c => [ #PH
         {
             Name => 'PreviewImageSize',
-            Condition => 'not $$self{MakerNoteSigma3}',
+            Condition => '$$self{MakerNoteSigmaVer} < 3',
             Notes => q{
                 PreviewImageStart for the SD1 and Merrill/Quattro models, and
                 PreviewImageSize for others
@@ -454,7 +483,7 @@ $VERSION = '1.28';
     0x001d => [ #PH
         {
             Name => 'MakerNoteVersion',
-            Condition => 'not $$self{MakerNoteSigma3}',
+            Condition => '$$self{MakerNoteSigmaVer} < 3',
             Notes => q{
                 PreviewImageLength for the SD1 and Merrill/Quattro models, and
                 MakerNoteVersion for others
@@ -473,7 +502,7 @@ $VERSION = '1.28';
     # 0x001e - int16u: 0, 4, 13 - flash mode for other models?
     0x001e => { #PH
         Name => 'PreviewImageSize',
-        Condition => '$$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
         Notes => 'only valid for some models',
         Writable => 'int16u',
         Count => 2,
@@ -483,7 +512,7 @@ $VERSION = '1.28';
     0x001f => [ #PH
         {
             Name => 'AFPoint', # (NC -- invalid for SD9,SD14?)
-            Condition => 'not $$self{MakerNoteSigma3}',
+            Condition => '$$self{MakerNoteSigmaVer} < 3',
             Notes => q{
                 MakerNoteVersion for the SD1 and Merrill/Quattro models, and AFPoint for
                 others
@@ -498,38 +527,49 @@ $VERSION = '1.28';
     # 0x0021 - string: " " for most models, or int8u[2]: '3 3' for the DP3 Merrill
     0x0022 => { #PH (NC)
         Name => 'FileFormat',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         # values: "JPG", "JPG-S", "JPG-P", "X3F", "X3F-S"
     },
     # 0x0023 - string: "", 10, 83, 131, 145, 150, 152, 169
     0x0024 => { # (invalid for SD9,SD14?)
         Name => 'Calibration',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
     },
     # 0x0025 - string: "", "0.70", "0.90"
     # 0x0026-2b - int32u: 0
     0x0026 => { #PH (NC)
         Name => 'FileFormat',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
+        # (also Sigma fp)
     },
-    0x0027 => { #PH
+    0x0027 => [{ #PH
         Name => 'LensType',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3 and $format eq "string"',
+        Notes => 'some newer models only',
         ValueConv => '$val =~ /^[0-9a-f]+$/i ? hex($val) : $val',
         # (truncate decimal part and convert hex)
         ValueConvInv => '$val=~s/\.\d+$//;$val=~/^0x/ and $val=hex($val);IsInt($val) ? sprintf("%x",$val) : $val',
         SeparateTable => 'LensType',
         PrintHex => 1,
         PrintConv => \%sigmaLensTypes,
-    },
+        PrintInt => 1,
+    },{ #PH
+        Name => 'LensType',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some other models like the fp',
+        Writable => 'int16u',
+        SeparateTable => 'LensType',
+        PrintHex => 1,
+        PrintConv => \%sigmaLensTypes,
+        PrintInt => 1,
+    }],
     0x002a => { #PH
         Name => 'LensFocalRange',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
         Writable => 'rational64u',
         Count => 2,
         PrintConv => '$val=~s/ / to /; $val',
@@ -539,13 +579,14 @@ $VERSION = '1.28';
         Name => 'LensMaxApertureRange',
         # for most models this gives the max aperture at the long/short focal lengths,
         # but for some models this gives the min/max aperture
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
         Writable => 'rational64u',
         Count => 2,
         PrintConv => '$val=~s/ / to /; $val',
         PrintConvInv => '$val=~s/to /; $val',
     },
+    # 0x002c is rational64u for some models, with a value that may be related to FNumber - PH
     0x002c => { #PH
         Name => 'ColorMode',
         Condition => '$format eq "int32u"',
@@ -570,7 +611,7 @@ $VERSION = '1.28';
     0x0030 => [ #PH
         {
             Name => 'LensApertureRange',
-            Condition => 'not $$self{MakerNoteSigma3}',
+            Condition => '$$self{MakerNoteSigmaVer} < 3',
             Notes => q{
                 Calibration for the SD1 and Merrill/Quattro models, and LensApertureRange
                 for others. Note that LensApertureRange changes with focal length, and some
@@ -582,7 +623,7 @@ $VERSION = '1.28';
     ],
     0x0031 => { #PH
         Name => 'FNumber',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         Writable => 'rational64u',
         PrintConv => 'sprintf("%.1f",$val)',
@@ -591,7 +632,7 @@ $VERSION = '1.28';
     },
     0x0032 => { #PH
         Name => 'ExposureTime',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         Writable => 'rational64u',
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
@@ -600,9 +641,8 @@ $VERSION = '1.28';
     },
     0x0033 => { #PH
         Name => 'ExposureTime2',
-        Condition => '$$self{Model} !~ / (SD1|SD9|SD15|Merrill|Quattro)$/',
+        Condition => '$$self{Model} !~ / (SD1|SD9|SD15|Merrill|Quattro|fp)$/',
         Notes => 'models other than the SD1, SD9, SD15 and Merrill/Quattro models',
-        Writable => 'string',
         ValueConv => '$val * 1e-6',
         ValueConvInv => 'int($val * 1e6 + 0.5)',
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
@@ -610,14 +650,14 @@ $VERSION = '1.28';
     },
     0x0034 => { #PH
         Name => 'BurstShot',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         Writable => 'int32u',
     },
     # 0x0034 - int32u: 0,1,2,3 or 4
     0x0035 => { #PH
         Name => 'ExposureCompensation',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         Writable => 'rational64s',
         # add a '+' sign to positive values
@@ -628,7 +668,7 @@ $VERSION = '1.28';
     # 0x0037-38 - string: ""
     0x0039 => { #PH (invalid for SD9, SD14?)
         Name => 'SensorTemperature',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         # (string format)
         PrintConv => 'IsInt($val) ? "$val C" : $val',
@@ -636,19 +676,19 @@ $VERSION = '1.28';
     },
     0x003a => { #PH
         Name => 'FlashExposureComp',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         Writable => 'rational64s',
     },
     0x003b => { #PH (how is this different from other Firmware?)
         Name => 'Firmware',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         Priority => 0,
     },
     0x003c => { #PH
         Name => 'WhiteBalance',
-        Condition => 'not $$self{MakerNoteSigma3}',
+        Condition => '$$self{MakerNoteSigmaVer} < 3',
         Notes => 'models other than the SD1 and Merrill/Quattro models',
         Priority => 0,
     },
@@ -658,13 +698,13 @@ $VERSION = '1.28';
     },
     0x0048 => { #PH
         Name => 'LensApertureRange',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
     },
     0x0049 => { #PH
         Name => 'FNumber',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
         Writable => 'rational64u',
         PrintConv => 'sprintf("%.1f",$val)',
         PrintConvInv => '$val',
@@ -672,8 +712,8 @@ $VERSION = '1.28';
     },
     0x004a => { #PH
         Name => 'ExposureTime',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
         Writable => 'rational64u',
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
         PrintConvInv => '$val',
@@ -682,15 +722,13 @@ $VERSION = '1.28';
     0x004b => [{ #PH
         Name => 'ExposureTime2',
         Condition => '$$self{Model} =~ /^SIGMA (SD1( Merrill)?|DP\d Merrill)$/',
-        Notes => 'SD1 and Merrill/Quattro models only',
-        Writable => 'string',
+        Notes => 'SD1 and DP Merrill models only',
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
         PrintConvInv => 'Image::ExifTool::Exif::ConvertFraction($val)',
-    },{
+    },{ #PH
         Name => 'ExposureTime2',
         Condition => '$$self{Model} =~ /^SIGMA dp\d Quattro$/i',
-        Notes => 'SD1 and Merrill/Quattro models only',
-        Writable => 'string',
+        Notes => 'DP Quattro models only',
         ValueConv => '$val / 1000000',
         ValueConvInv => '$val * 1000000',
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
@@ -698,8 +736,8 @@ $VERSION = '1.28';
     }],
     0x004d => { #PH
         Name => 'ExposureCompensation',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
         Writable => 'rational64s',
         # add a '+' sign to positive values
         PrintConv => '$val and $val =~ s/^(\d)/\+$1/; $val',
@@ -708,45 +746,105 @@ $VERSION = '1.28';
     # 0x0054 - string: "F20","F23"
     0x0055 => { #PH
         Name => 'SensorTemperature',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
         # (string format)
         PrintConv => 'IsInt($val) ? "$val C" : $val',
         PrintConvInv => '$val=~s/ ?C$//; $val',
     },
     0x0056 => { #PH (NC)
         Name => 'FlashExposureComp',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
         Writable => 'rational64s',
     },
     0x0057 => { #PH (how is this different from other Firmware?)
-        Name => 'Firmware',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Name => 'Firmware2',
+        Condition => '$format eq "string"',
+        Notes => 'some newer models only',
         Priority => 0,
     },
     0x0058 => { #PH
         Name => 'WhiteBalance',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
         Priority => 0,
     },
     0x0059 => { #PH
         Name => 'DigitalFilter',
-        Condition => '$$self{MakerNoteSigma3}',
-        Notes => 'SD1 and Merrill/Quattro models only',
-        # seen: Standard, Landscape,Monochrome,Neutral,Portrait,Sepia,Vivid
+        Condition => '$$self{MakerNoteSigmaVer} >= 3',
+        Notes => 'some newer models only',
+        # seen: Standard,Landscape,Monochrome,Neutral,Portrait,Sepia,Vivid
     },
     # 0x005a/b/c - rational64s: 0/10 for the SD1
-    0x0084 => { #PH
+    0x0084 => { #PH (Quattro models and fp)
         Name => 'Model',
         Description => 'Camera Model Name',
-        Writable => 'string',
     },
     # 0x0085
+    0x0086 => { #PH (Quattro models)
+        Name => 'ISO',
+        Writable => 'int16u',
+    },
+    0x0087 => 'ResolutionMode', #PH (Quattro models)
+    0x0088 => 'WhiteBalance', #PH (Quattro models)
+    0x008c => 'Firmware', #PH (Quattro models)
+    0x011f => { #IB (FP DNG images)
+        Name => 'CameraCalibration',
+        Writable => 'float',
+        Count => 9,
+    },
+    0x0120 => { #IB (FP DNG images)
+        Name => 'WBSettings',
+        SubDirectory => { TagTable => 'Image::ExifTool::Sigma::WBSettings' },
+    },
+    0x0121 => { #IB (FP DNG images)
+        Name => 'WBSettings2',
+        SubDirectory => { TagTable => 'Image::ExifTool::Sigma::WBSettings2' },
+    },
 );
 
+# WB settings (ref IB)
+%Image::ExifTool::Sigma::WBSettings = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FORMAT => 'float',
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    0  => { Name => 'WB_RGBLevelsAuto',         Format => 'float[3]' },
+    3  => { Name => 'WB_RGBLevelsDaylight',     Format => 'float[3]' },
+    6  => { Name => 'WB_RGBLevelsShade',        Format => 'float[3]' },
+    9  => { Name => 'WB_RGBLevelsOvercast',     Format => 'float[3]' },
+    12 => { Name => 'WB_RGBLevelsIncandescent', Format => 'float[3]' },
+    15 => { Name => 'WB_RGBLevelsFluorescent',  Format => 'float[3]' },
+    18 => { Name => 'WB_RGBLevelsFlash',        Format => 'float[3]' },
+    21 => { Name => 'WB_RGBLevelsCustom1',      Format => 'float[3]' },
+    24 => { Name => 'WB_RGBLevelsCustom2',      Format => 'float[3]' },
+    27 => { Name => 'WB_RGBLevelsCustom3',      Format => 'float[3]' },
+);
+
+# WB settings (ref IB)
+%Image::ExifTool::Sigma::WBSettings2 = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FORMAT => 'float',
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    0  => { Name => 'WB_RGBLevelsUnknown0', Unknown => 1, Format => 'float[3]' },
+    3  => { Name => 'WB_RGBLevelsUnknown1', Unknown => 1, Format => 'float[3]' },
+    6  => { Name => 'WB_RGBLevelsUnknown2', Unknown => 1, Format => 'float[3]' },
+    9  => { Name => 'WB_RGBLevelsUnknown3', Unknown => 1, Format => 'float[3]' },
+    12 => { Name => 'WB_RGBLevelsUnknown4', Unknown => 1, Format => 'float[3]' },
+    15 => { Name => 'WB_RGBLevelsUnknown5', Unknown => 1, Format => 'float[3]' },
+    18 => { Name => 'WB_RGBLevelsUnknown6', Unknown => 1, Format => 'float[3]' },
+    21 => { Name => 'WB_RGBLevelsUnknown7', Unknown => 1, Format => 'float[3]' },
+    24 => { Name => 'WB_RGBLevelsUnknown8', Unknown => 1, Format => 'float[3]' },
+    27 => { Name => 'WB_RGBLevelsUnknown9', Unknown => 1, Format => 'float[3]' },
+);
+
+
 1;  # end
 
 __END__
@@ -766,7 +864,7 @@ Sigma and Foveon maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
